import { APIRoute, MiddlewareRoute, RouteService, Method } from "../../route";
import { ExpressService } from "../expressService";
import { UserService } from "../userService";
import { UserDefinitionService, User } from "../userDefinitionService";
import { RouteSessionService } from "../routeSessionService";
import * as express from "express";
import * as Leaf from "leaf-ts";
export declare const Errors: Leaf.ErrorsOf<{
    Forbidden: {};
    AuthorizationFailed: {};
}>;
export declare class UserAPIService<TUser extends User = User> extends RouteService {
    readonly name: string;
    dependencies: string[];
    prefix: string;
    services: {
        ExpressService: ExpressService;
        RouteSessionService: RouteSessionService;
        UserService: UserService<TUser>;
        UserDefinitionService: UserDefinitionService<TUser>;
    };
    initialize(): void;
}
export declare class SessionUserProvider extends MiddlewareRoute {
    services: {
        UserService: UserService<User>;
        RouteSessionService: RouteSessionService;
    };
    req: express.Request & {
        session: {
            username: string;
        };
    };
    info: {
        user: User;
    };
    handle(): void;
}
export declare class Register extends APIRoute<null> {
    readonly method: Method<"POST">;
    readonly path: string;
    readonly minUsernameLength: number;
    readonly maxUsernameLength: number;
    readonly minPasswordLength: number;
    readonly maxPasswordLength: number;
    services: {
        UserService: UserService<User>;
        RouteSessionService: RouteSessionService;
        UserDefinitionService: UserDefinitionService<User>;
    };
    body: Partial<User>;
    private checkUserValidation(user);
    req: express.Request & {
        session: {
            username: string;
        };
    };
    info: {
        user: User;
    };
    handle(): void;
}
export declare class GetUserProfile extends APIRoute<User> {
    readonly method: Method<"GET">;
    readonly path: string;
    services: {
        UserService: UserService<User>;
    };
    info: {
        user: User;
    };
    handle(): void;
}
export declare class Login extends APIRoute<User> {
    readonly method: Method<"PUT">;
    readonly path: string;
    services: {
        UserService: UserService<User>;
    };
    body: {
        login: string;
        password: string;
    };
    req: express.Request & {
        session: {
            username: string;
        };
    };
    handle(): void;
}
export declare class Logout extends APIRoute<null> {
    readonly method: Method<"DELETE">;
    readonly path: string;
    req: express.Request & {
        session: {
            username: string;
        };
    };
    services: {
        UserService: UserService<User>;
    };
    handle(): void;
}
