"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var route_1 = require("../../route");
var Leaf = require("leaf-ts");
exports.Errors = Leaf.ErrorDoc.create()
    .define("Forbidden")
    .define("AuthorizationFailed")
    .generate();
var UserAPIService = (function (_super) {
    __extends(UserAPIService, _super);
    function UserAPIService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.name = "api.user";
        _this.dependencies = ["ExpressService", "RouteSessionService", "UserService", "UserDefinitionService"];
        _this.prefix = "/api/";
        return _this;
    }
    UserAPIService.prototype.initialize = function () {
        this.addRoutes(module.exports);
        this.installTo(this.services.ExpressService.server);
    };
    return UserAPIService;
}(route_1.RouteService));
exports.UserAPIService = UserAPIService;
var SessionUserProvider = (function (_super) {
    __extends(SessionUserProvider, _super);
    function SessionUserProvider() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SessionUserProvider.prototype.handle = function () {
        var _this = this;
        var session = this.req.session;
        if (!session) {
            this.next();
        }
        var name = this.req.session.username;
        if (!name) {
            this.next();
            return;
        }
        this.services.UserService.getUserByUsername(name, function (err, user) {
            if (err) {
                _this.next();
                return;
            }
            _this.info.user = user;
            _this.next();
        });
    };
    return SessionUserProvider;
}(route_1.MiddlewareRoute));
exports.SessionUserProvider = SessionUserProvider;
var Register = (function (_super) {
    __extends(Register, _super);
    function Register() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.method = route_1.Method.POST;
        _this.path = "user";
        _this.minUsernameLength = 3;
        _this.maxUsernameLength = 32;
        _this.minPasswordLength = 8;
        _this.maxPasswordLength = 1024;
        return _this;
    }
    Register.prototype.checkUserValidation = function (user) {
        return this.services.UserDefinitionService.UserValidator.check(user);
    };
    Register.prototype.handle = function () {
        var _this = this;
        var userToCheck = this.body;
        try {
            this.checkUserValidation(userToCheck);
        }
        catch (err) {
            this.error(err);
            return;
        }
        this.services.UserService.createUser(userToCheck, function (err, user) {
            if (err) {
                _this.error(err);
                return;
            }
            _this.req.session.username = user.username;
            _this.success(null);
        });
    };
    return Register;
}(route_1.APIRoute));
exports.Register = Register;
var GetUserProfile = (function (_super) {
    __extends(GetUserProfile, _super);
    function GetUserProfile() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.method = route_1.Method.GET;
        _this.path = "user";
        return _this;
    }
    GetUserProfile.prototype.handle = function () {
        if (!this.info.user) {
            this.error(new exports.Errors.Forbidden);
            return;
        }
        var user = this.info.user;
        this.success({
            username: user.username,
            email: user.email,
            displayName: user.displayName
        });
    };
    return GetUserProfile;
}(route_1.APIRoute));
exports.GetUserProfile = GetUserProfile;
var Login = (function (_super) {
    __extends(Login, _super);
    function Login() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.method = route_1.Method.PUT;
        _this.path = "user";
        return _this;
    }
    Login.prototype.handle = function () {
        var _this = this;
        var loginUser = this.body;
        this.services.UserService.getUserByLogin(loginUser, function (err, user) {
            if (err) {
                _this.error(err);
                return;
            }
            if (!user) {
                _this.error(new exports.Errors.AuthorizationFailed);
                return;
            }
            _this.req.session.username = user.username;
            _this.success(user);
        });
    };
    return Login;
}(route_1.APIRoute));
exports.Login = Login;
var Logout = (function (_super) {
    __extends(Logout, _super);
    function Logout() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.method = route_1.Method.DELETE;
        _this.path = "user";
        return _this;
    }
    Logout.prototype.handle = function () {
        this.req.session.username = null;
        this.success(null);
    };
    return Logout;
}(route_1.APIRoute));
exports.Logout = Logout;
//# sourceMappingURL=api.user.js.map