import { Service } from "../service";
import { UserDefinitionService, User } from "./userDefinitionService";
import { MongodbService } from "./mongodbService";
import * as mongodb from "mongodb";
import * as Leaf from "leaf-ts";
export declare type Callback<T> = (err?: Error, result?: T) => void;
export declare const Errors: Leaf.ErrorsOf<{
    UnknownError: {};
    DBError: {};
    AlreadyExists: {};
    InvalidParameter: {};
    UsernameAlreadyExists: {};
    EmailAlreadyExists: {};
}>;
export declare class UserService<TUser extends User = User> extends Service {
    readonly name: string;
    readonly dependencies: any;
    readonly collectionName: string;
    readonly Errors: Leaf.ErrorsOf<{
        UnknownError: {};
        DBError: {};
        AlreadyExists: {};
        InvalidParameter: {};
        UsernameAlreadyExists: {};
        EmailAlreadyExists: {};
    }>;
    constructor(config?: {
        user?: {
            salt?: string;
            blacklist?: string[];
        };
    });
    private blacklist;
    private salt;
    services: {
        MongodbService: MongodbService;
        UserDefinitionService: UserDefinitionService<TUser>;
    };
    userCollection: mongodb.Collection;
    initialize(): void;
    private createPasswordHashString(rawPassword);
    private translateMongoId(doc);
    toPublicUser(user: TUser): TUser;
    getUserById(id: string, callback: Callback<TUser>): void;
    getUserByUsername(username: string, callback: Callback<TUser>): void;
    getUserByEmail(email: string, callback: Callback<TUser>): void;
    getUserByLogin(option: {
        login: string;
        password: string;
    }, callback: Callback<TUser>): void;
    createUser(partialUser: Partial<TUser>, callback: Callback<TUser>): void;
    updateUserPasswordByUsername(option: {
        username: string;
        password: string;
    }, callback: Callback<TUser>): void;
    updateUserPasswordById(option: {
        id: string;
        password: string;
    }, callback: Callback<TUser>): void;
}
