"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var service_1 = require("../service");
var mongodb = require("mongodb");
var crypto = require("crypto");
var Leaf = require("leaf-ts");
exports.Errors = Leaf.ErrorDoc.create()
    .define("UnknownError")
    .define("DBError")
    .define("AlreadyExists")
    .define("UsernameAlreadyExists")
    .define("EmailAlreadyExists")
    .define("InvalidParameter")
    .generate();
function clearUserPassword(obj, name, descriptor) {
    var fn = descriptor.value;
    descriptor.value = function (option, callback) {
        fn.call(this, option, function (err, value) {
            if (value instanceof Array) {
                value = value.forEach(function (item) {
                    delete item.passwordHash;
                });
            }
            else if (value) {
                delete value.passwordHash;
            }
            callback(err, value);
        });
    };
}
var UserService = (function (_super) {
    __extends(UserService, _super);
    function UserService(config) {
        if (config === void 0) { config = {}; }
        var _this = _super.call(this) || this;
        _this.name = "UserService";
        _this.dependencies = ["MongodbService", "UserDefinitionService"];
        _this.collectionName = "user";
        _this.Errors = exports.Errors;
        _this.blacklist = [];
        _this.salt = "default-user-secret-salt";
        if (config.user) {
            if (config.user.salt) {
                _this.salt = config.user.salt;
            }
            if (config.user.blacklist) {
                _this.blacklist = config.user.blacklist;
            }
        }
        return _this;
    }
    UserService.prototype.initialize = function () {
        this.salt;
        this.userCollection = this.services.MongodbService.db.collection(this.collectionName);
        this.userCollection.createIndex({
            username: 1
        }, {
            unique: true
        });
        this.userCollection.createIndex({
            email: 1
        }, {
            unique: true
        });
        this.userCollection.createIndex({
            passwordHash: 1
        });
    };
    UserService.prototype.createPasswordHashString = function (rawPassword) {
        var level1 = crypto.createHash("sha256").update(rawPassword + this.salt).digest('hex');
        var level2 = crypto.createHash("sha512").update(level1 + this.salt).digest('hex');
        return level2;
    };
    UserService.prototype.translateMongoId = function (doc) {
        if (!doc)
            return null;
        var id = doc._id;
        delete doc._id;
        doc.id = id.toString();
        return doc;
    };
    UserService.prototype.toPublicUser = function (user) {
        delete user.passwordHash;
        delete user.password;
        return user;
    };
    UserService.prototype.getUserById = function (id, callback) {
        var _this = this;
        var _id = new mongodb.ObjectID(id);
        this.userCollection.findOne({ _id: _id }, function (error, doc) {
            if (error) {
                callback(new exports.Errors.DBError("DatabaseError", { via: error }));
                return;
            }
            var user = _this.translateMongoId(doc);
            callback(null, user);
        });
    };
    UserService.prototype.getUserByUsername = function (username, callback) {
        var _this = this;
        username = username.toLowerCase();
        this.userCollection.findOne({ username: username }, function (error, doc) {
            if (error) {
                callback(new exports.Errors.DBError("DatabaseError", { via: error }));
                return;
            }
            var user = _this.translateMongoId(doc);
            callback(null, user);
        });
    };
    UserService.prototype.getUserByEmail = function (email, callback) {
        var _this = this;
        email = email.toLowerCase();
        this.userCollection.findOne({ email: email }, function (error, doc) {
            if (error) {
                callback(new exports.Errors.DBError("DatabaseError", { via: error }));
                return;
            }
            var user = _this.translateMongoId(doc);
            callback(null, user);
        });
    };
    UserService.prototype.getUserByLogin = function (option, callback) {
        var _this = this;
        var login = option.login.toLowerCase();
        var passwordHash = this.createPasswordHashString(option.password);
        this.userCollection.findOne({ passwordHash: passwordHash, $or: [{ username: login }, { email: login }] }, function (error, doc) {
            if (error) {
                callback(new exports.Errors.DBError("DatabaseError", { via: error }));
                return;
            }
            var user = _this.translateMongoId(doc);
            callback(null, user);
        });
    };
    UserService.prototype.createUser = function (partialUser, callback) {
        var _this = this;
        if (this.blacklist.indexOf(partialUser.username) >= 0) {
            callback(new exports.Errors.InvalidParameter("Invalid username"));
            return;
        }
        if (!partialUser.username || !partialUser.email) {
            callback(new exports.Errors.InvalidParameter("Empty username or email"));
            return;
        }
        // use username as display name by default
        if (!partialUser.displayName) {
            partialUser.displayName = partialUser.username;
        }
        // then convert username to all lowercase
        partialUser.username = partialUser.username.toLocaleLowerCase();
        partialUser.email = partialUser.email.toLowerCase();
        try {
        }
        catch (error) {
            callback(error);
            return;
        }
        try {
            this.services.UserDefinitionService.UserValidator.check(partialUser);
        }
        catch (e) {
            callback(new exports.Errors.InvalidParameter());
            return;
        }
        var passwordHash = this.createPasswordHashString(partialUser.password);
        var user = {
            username: partialUser.username,
            displayName: partialUser.displayName,
            passwordHash: passwordHash,
            email: partialUser.email,
            crteateAt: Date.now()
        };
        this.getUserByUsername(user.username, function (error, doc) {
            if (doc) {
                callback(new exports.Errors.UsernameAlreadyExists());
                return;
            }
            _this.getUserByEmail(user.email, function (error, doc) {
                if (doc) {
                    callback(new exports.Errors.EmailAlreadyExists());
                    return;
                }
                _this.userCollection.insert(user, function (error, doc) {
                    if (error) {
                        // on a edge case that 2 user try to use a same username and something like that
                        callback(new exports.Errors.DBError("DatabaseError", { via: error }));
                        return;
                    }
                    var user = _this.translateMongoId(doc.ops[0]);
                    callback(null, user);
                });
            });
        });
    };
    UserService.prototype.updateUserPasswordByUsername = function (option, callback) {
        var _this = this;
        try {
            this.services.UserDefinitionService.UserValidator.check({ password: option.password });
        }
        catch (e) {
            callback(new exports.Errors.InvalidParameter());
            return;
        }
        var username = option.username.toLowerCase();
        var passwordHash = this.createPasswordHashString(option.password);
        this.userCollection.findOneAndUpdate({ username: username }, { $set: { passwordHash: passwordHash } }, { returnOriginal: false }, function (error, doc) {
            if (error) {
                callback(error);
                return;
            }
            var user = _this.translateMongoId(doc.value);
            callback(null, user);
        });
    };
    UserService.prototype.updateUserPasswordById = function (option, callback) {
        var _this = this;
        var _id = new mongodb.ObjectID(option.id);
        try {
            this.services.UserDefinitionService.UserValidator.check({ password: option.password });
        }
        catch (e) {
            callback(new exports.Errors.InvalidParameter());
            return;
        }
        var passwordHash = this.createPasswordHashString(option.password);
        this.userCollection.findOneAndUpdate({ _id: _id }, { $set: { passwordHash: passwordHash } }, { returnOriginal: false }, function (error, doc) {
            if (error) {
                callback(error);
                return;
            }
            var user = _this.translateMongoId(doc.value);
            callback(null, user);
        });
    };
    __decorate([
        clearUserPassword
    ], UserService.prototype, "getUserById", null);
    __decorate([
        clearUserPassword
    ], UserService.prototype, "getUserByUsername", null);
    __decorate([
        clearUserPassword
    ], UserService.prototype, "getUserByEmail", null);
    __decorate([
        clearUserPassword
    ], UserService.prototype, "getUserByLogin", null);
    __decorate([
        clearUserPassword
    ], UserService.prototype, "createUser", null);
    __decorate([
        clearUserPassword
    ], UserService.prototype, "updateUserPasswordByUsername", null);
    __decorate([
        clearUserPassword
    ], UserService.prototype, "updateUserPasswordById", null);
    return UserService;
}(service_1.Service));
exports.UserService = UserService;
//# sourceMappingURL=userService.js.map