"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("source-map-support/register");
var DependencyManager = (function () {
    function DependencyManager() {
        this.context = {};
        this.onAddDymanicDependency = function (name) { };
    }
    DependencyManager.prototype.add = function (name, deps) {
        if (this.context[name])
            throw new Error("Duplicate dependency entry: " + name);
        if (deps && deps.length == 0)
            deps = null;
        this.context[name] = deps;
    };
    DependencyManager.prototype.resolveDependency = function (name, chain) {
        if (chain === void 0) { chain = []; }
        if (typeof this.context[name] === "undefined") {
            this.onAddDymanicDependency(name);
            if (typeof this.context[name] === "undefined") {
                throw new Error(name + " required by " + chain[chain.length - 1] + " couldn't be fullfilled.");
            }
        }
        if (this.context[name] === null)
            return [];
        chain = chain.slice();
        if (chain.indexOf(name) >= 0) {
            chain.push(name);
            throw new Error("Recursive dependency detected: " + chain.join("\n=>"));
        }
        chain.push(name);
        var result = [];
        var all = [];
        for (var _i = 0, _a = this.context[name] || []; _i < _a.length; _i++) {
            var matcher = _a[_i];
            if (typeof matcher === "string") {
                if (all.indexOf(matcher) < 0)
                    all.push(matcher);
            }
            else if (matcher instanceof RegExp) {
                for (var name_1 in this.context) {
                    if (matcher.test(name_1) && all.indexOf(name_1) < 0) {
                        all.push(name_1);
                    }
                }
            }
        }
        for (var _b = 0, all_1 = all; _b < all_1.length; _b++) {
            var dep = all_1[_b];
            var deps = this.resolveDependency(dep, chain);
            for (var _c = 0, deps_1 = deps; _c < deps_1.length; _c++) {
                var subDep = deps_1[_c];
                if (result.indexOf(subDep) < 0)
                    result.push(subDep);
            }
            result.push(dep);
        }
        return result;
    };
    DependencyManager.prototype.resolve = function () {
        var result = [];
        for (var name_2 in this.context) {
            var deps = this.resolveDependency(name_2);
            for (var _i = 0, deps_2 = deps; _i < deps_2.length; _i++) {
                var dep = deps_2[_i];
                if (result.indexOf(dep) < 0)
                    result.push(dep);
            }
            if (result.indexOf(name_2) < 0)
                result.push(name_2);
        }
        return result;
    };
    return DependencyManager;
}());
exports.DependencyManager = DependencyManager;
//# sourceMappingURL=dependency.js.map